'use strict';

const fs = require('fs');
const R = require('ramda');
const inquirer = require('inquirer');
const prettyBytes = require('pretty-bytes');

const apiWrapper = require('./api-wrapper');
const tools = require('../utils/tools');
const profileHelper = require('../utils/profile-helper');
const jsonUtility = require('../utils/json-utility');
const CONSTANTS = require('../utils/constants');
const optionValidator = require('../commands/option-validator');

const LIST_QUERY_PARAMS = [
    "maxResults",
    "sortDirection",
    "nextToken"
];

//Public
module.exports = {
    createCommand: (commander) => {
        buildCreateModelCatalogCommand(commander);
        buildCreateModelCatalogVersionCommand(commander);
        buildUpdateModelCatalogVersionCommand(commander);
        buildUpdateModelCatalogCommand(commander);
        buildGetModelCatalogVersionCommand(commander);
        buildGetModelCatalogCommand(commander);
        buildListModelCatalogsCommand(commander);
        buildListModelCatalogVersionsCommand(commander);
        buildGetModelCatalogValuesCommand(commander);
        buildGetModelCatalogUpdateStatusCommand(commander);
        buildDeleteModelCatalogCommand(commander);
        buildDeleteModelCatalogVersionCommand(commander);
    }
};

function buildCreateModelCatalogCommand(commander) {
    commander
        .command('create-model-catalog')
        .usage('<-n|--catalog-name <catalog-name>> [-d|--catalog-description <catalog-description>] [-p|--profile <profile>] [--debug]')
        .description('create a new model catalog.')
        .option('-n, --catalog-name <catalog-name>', 'name of the catalog')
        .option('-d, --catalog-description <catalog-description>', 'description of the catalog')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        
        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        if (!vendorId) {
            console.error('[Error]: cannot find the vendor-id associated with the current profile.');
            process.exit(1);
        }
        
        try {
            optionValidator.validateModelCatalogName(options.catalogName);
            optionValidator.validateDescription(options.catalogDescription, { required: false });
        } catch (err) {
            console.error(`[Error]: ${err}`);
            process.exit(1);
        }
        
        apiWrapper.callCreateModelCatalog(options.catalogName, options.description, vendorId, profile, options.debug, (data) => {
            const response = tools.convertDataToJsonObject(data.body);
            if (response) {
                console.log(JSON.stringify(response, null, 2));
            }
        });
    }
}

function buildCreateModelCatalogVersionCommand(commander) {
    commander
        .command('create-model-catalog-version')
        .usage('<-c|--catalog-id <catalog-id>> <-f|--file <file-path>> [-d|--version-description <version-description>] [-p|--profile <profile>] [--debug]')
        .description('create a new model catalog version')
        .option('-c, --catalog-id <catalog-id>', 'id of the catalog')
        .option('-f, --file <file-path>', 'file for source')
        .option('-d, --version-description <version-description>', 'description for the new version')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        
        if (!options.file) {
            console.warn('Please input required parameter: file.');
            return;
        }
        
        try {
            optionValidator.validateModelCatalogId(options.catalogId);
            optionValidator.validateDescription(options.versionDescription, { required: false });
        } catch (err) {
            console.error(`[Error]: ${err}`);
            process.exit(1);
        }
        
        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        const source = jsonUtility.read(options.file);
        
        if (!vendorId) {
            console.error('[Error]: cannot find the vendor-id associated with the current profile.');
            process.exit(1);
        }
        apiWrapper.callCreateModelCatalogVersion(options.catalogId, source, options.versionDescription, profile, options.debug, (data) => {
            const response = tools.convertDataToJsonObject(data);
            if (response.headers.location) {
                console.log("catalog version submitted");
                console.log("Please use the following command to track the to track the create version status "+
                        "\n get-model-catalog-update-status -c " + options.catalogId + " -u "
                        + tools.retrieveUpdateRequestId(response.headers.location));
            }
        });
    }
}

function buildUpdateModelCatalogVersionCommand(commander) {
    commander
        .command('update-model-catalog-version')
        .usage('<-c|--catalog-id <catalog-id>> <--catalog-version <catalog-version>> [-d|--version-description <--version-description>] [-p|--profile <profile>] [--debug]')
        .description('update model catalog version.')
        .option('-c, --catalog-id <catalog-id>', 'catalog identifier')
        .option('--catalog-version <catalog-version>', 'version of the catalog')
        .option('-d, --version-description <version-description>', 'description of this version')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        if (!vendorId) {
            console.error('[Error]: cannot find the vendor-id associated with the current profile.');
            process.exit(1);
        }
        
        try {
            optionValidator.validateModelCatalogId(options.catalogId);
            optionValidator.validateCatalogVersion(options.catalogVersion);
            optionValidator.validateDescription(options.versionDescription, { required: false });
        } catch (err) {
            console.error(`[Error]: ${err}`);
            process.exit(1);
        }
        
        apiWrapper.callUpdateModelCatalogVersion(options.catalogId, options.catalogVersion, options.versionDescription, profile, options.debug, (data) => {
        	console.log('Model catalog version updated successfully.');
        });
    }
}

function buildUpdateModelCatalogCommand(commander) {
    commander
        .command('update-model-catalog')
        .usage('<-c|--catalog-id <catalog-id>> [-d|--catalog-description <catalog-description>] [-n |--catalog-name [catalog-name]] [-p|--profile <profile>] [--debug]')
        .description('update model catalog.')
        .option('-c, --catalog-id <catalog-id>', 'catalog identifier')
        .option('-d, --catalog-description <catalog-description>', 'description of the catalog')
        .option('-n, --catalog-name <catalog-name>', 'name of the catalog')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        if (!vendorId) {
            console.error('[Error]: cannot find the vendor-id associated with the current profile.');
            process.exit(1);
        }
        
        try {
            optionValidator.validateModelCatalogId(options.catalogId);
            optionValidator.validateDescription(options.catalogDescription, { required: false });
            optionValidator.validateModelCatalogName(options.catalogName, { required: false } );
        } catch (err) {
            console.error(`[Error]: ${err}`);
            process.exit(1);
        }
        
        apiWrapper.callUpdateModelCatalog(options.catalogName, options.catalogId, options.catalogDescription, profile, options.debug, (data) => {
             console.log('Model catalog updated successfully.');
        });
    }
}

function buildGetModelCatalogVersionCommand(commander) {
    commander
        .command('get-model-catalog-version')
        .usage('<-c|--catalog-id <catalog-id>> <--catalog-version <catalog-version>> [-p|--profile <profile>] [--debug]')
        .description('get an existing catalog version')
        .option('-c, --catalog-id <catalog-id>', 'catalog identifier')
        .option('--catalog-version <catalog-version>', 'catalog version')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        if (!vendorId) {
            console.error('[Error]: cannot find the vendor-id associated with the current profile.');
            process.exit(1);
        }
        
        try {
            optionValidator.validateModelCatalogId(options.catalogId);
            optionValidator.validateCatalogVersion(options.catalogVersion);
        } catch (err) {
            console.error(`[Error]: ${err}`);
            process.exit(1);
        }
        
        apiWrapper.callGetModelCatalogVersion(options.catalogId, options.catalogVersion, profile, options.debug, (data) => {
            const response = tools.convertDataToJsonObject(data.body);
            if (response) {
                console.log(JSON.stringify(response, null, 2));
            }
        });
    }
}

function buildGetModelCatalogCommand(commander) {
    commander
        .command('get-model-catalog')
        .usage('<-c|--catalog-id <catalog-id>> [-p|--profile <profile>] [--debug]')
        .description('get an existing model catalog.')
        .option('-c, --catalog-id <catalog-id>', 'catalog identifier')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        if (!vendorId) {
            console.error('[Error]: cannot find the vendor-id associated with the current profile.');
            process.exit(1);
        }
        
        try {
            optionValidator.validateModelCatalogId(options.catalogId);
        } catch (err) {
            console.error(`[Error]: ${err}`);
            process.exit(1);
        }
        
        apiWrapper.callGetModelCatalog(options.catalogId, profile, options.debug, (data) => {
            const response = tools.convertDataToJsonObject(data.body);
            if (response) {
                console.log(JSON.stringify(response, null, 2));
            }
        });
    }
}

function buildListModelCatalogsCommand(commander) {
    commander
        .command('list-model-catalogs')
        .usage('[-m|--max-results <max-results>] [--max-results <max-results>] [--next-token <next-token>] [--sort-direction <sort-direction>] [-p|--profile <profile>] [--debug]')
        .description('list all model catalog for this profile')
        .option('--max-results <max-results>', 'max results returned by the request')
        .option('--next-token <next-token>', 'next token if the result is paginated')
        .option('--sort-direction <sort-direction>', 'order to sort results')
        .option('-p, --profile <profile[', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        if (!vendorId) {
            console.error('[Error]: cannot find the vendor-id associated with the current profile.');
            process.exit(1);
        }
        
        try {
            optionValidator.validateNextToken(options.nextToken, { required: false });
            optionValidator.validateMaxResults(options.maxResult, { required: false });
            optionValidator.validateSortDirectionForListModelCatalog(options.maxResult, { required: false });
        } catch (err) {
            console.error(`[Error]: ${err}`);
            process.exit(1);
        }
        
        const queryParameters = {};
        LIST_QUERY_PARAMS.forEach(optionName => {
            if (options[optionName]) {
                queryParameters[optionName] = options[optionName];
            }
        });
        queryParameters["vendorId"] = vendorId;
        apiWrapper.callListModelCatalogs(queryParameters, profile, options.debug, (data) => {
            const response = tools.convertDataToJsonObject(data.body);
            if (response) {
                console.log(JSON.stringify(removeLinksFromResponse(response), null, 2));
            }
        });
    }
}

function buildListModelCatalogVersionsCommand(commander) {
    commander
        .command('list-model-catalog-versions')
        .usage('<-c|--catalog-id <catalog-id>> [--max-results <max-results>] [--next-token <next-token>] [--sort-direction <sort-direction>] [-p|--profile <profile>] [--debug]')
        .description('list all model catalog versions for the given catalog-id')
        .option('-c, --catalog-id <catalog-id>', 'catalog identifier')
        .option('--max-results <max-results>', 'max results returned by the request')
        .option('--next-token <next-token>', 'next token if the result is paginated')
        .option('--sort-direction <sort-direction>', 'order to sort results')
        .option('-p, --profile <profile[', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        if (!vendorId) {
            console.error('[Error]: cannot find the vendor-id associated with the current profile.');
            process.exit(1);
        }
        
        try {
            optionValidator.validateModelCatalogId(options.catalogId);
            optionValidator.validateNextToken(options.nextToken, { required: false });
            optionValidator.validateMaxResults(options.maxResult, { required: false });
            optionValidator.validateSortDirectionForListModelCatalog(options.sortDirection, { required: false });
        } catch (err) {
            console.error(`[Error]: ${err}`);
            process.exit(1);
        }
        
        const queryParameters = {};
        LIST_QUERY_PARAMS.forEach(optionName => {
            if (options[optionName]) {
                queryParameters[optionName] = options[optionName];
            }
        });
        
        apiWrapper.callListModelCatalogVersions(options.catalogId, queryParameters, profile, options.debug, (data) => {
            const response = tools.convertDataToJsonObject(data.body);
            if (response) {
                console.log(JSON.stringify(removeLinksFromResponse(response), null, 2));
            }
        });
    }
}

function buildGetModelCatalogValuesCommand(commander) {
    commander
        .command('get-model-catalog-values')
        .usage('<-c|--catalog-id <catalog-id>>  <--catalog-version <catalog-version>> [--max-results <max-results>] [--next-token <next-token>] [--sort-direction <sort-direction>] [-p|--profile <profile>] [--debug]')
        .description('get catalog values for this profile')
        .option('-c, --catalog-id <catalog-id>', 'catalog identifier')
        .option('--catalog-version <catalog-version>', 'catalog version')
        .option('--max-results <max-results>', 'max results returned by the request')
        .option('--next-token <next-token>', 'next token if the result is paginated')
        .option('--sort-direction <sort-direction>', 'order to sort results')
        .option('-p, --profile <profile[', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        if (!vendorId) {
            console.error('[Error]: cannot find the vendor-id associated with the current profile.');
            process.exit(1);
        }
        
        try {
            optionValidator.validateModelCatalogId(options.catalogId);
            optionValidator.validateCatalogVersion(options.catalogVersion);
            optionValidator.validateNextToken(options.nextToken, { required: false });
            optionValidator.validateMaxResults(options.maxResult, { required: false });
            optionValidator.validateSortDirectionForListModelCatalog(options.maxResult, { required: false });
        } catch (err) {
            console.error(`[Error]: ${err}`);
            process.exit(1);
        }
        
        const queryParameters = {};
        LIST_QUERY_PARAMS.forEach(optionName => {
            if (options[optionName]) {
                queryParameters[optionName] = options[optionName];
            }
        });
        
        apiWrapper.callGetModelCatalogValues(options.catalogId, options.catalogVersion, queryParameters, profile, options.debug, (data) => {
            const response = tools.convertDataToJsonObject(data.body);
            if (response) {
                console.log(JSON.stringify(removeLinksFromResponse(response), null, 2));
            }
        });
    }
}

function buildGetModelCatalogUpdateStatusCommand(commander) {
    commander
        .command('get-model-catalog-update-status')
        .usage('<-c|--catalog-id <catalog-id>>  <-u|--update-request-id <update-request-id>> [-p|--profile <profile>] [--debug]')
        .description('get update status for model catalog.')
        .option('-c, --catalog-id <catalog-id>', 'catalog identifier')
        .option('-u, --update-request-id <update-request-id>', 'catalog version update request id')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        if (!vendorId) {
            console.error('[Error]: cannot find the vendor-id associated with the current profile.');
            process.exit(1);
        }
        
        try {
            optionValidator.validateModelCatalogId(options.catalogId);
            optionValidator.validateModelCatalogupdateRequestId(options.updateRequestId);
        } catch (err) {
            console.error(`[Error]: ${err}`);
            process.exit(1);
        }
        
        apiWrapper.callGetModelCatalogUpdateStatus(options.catalogId, options.updateRequestId, profile, options.debug, (data) => {
            const response = tools.convertDataToJsonObject(data.body);
            if (response) {
                console.log(JSON.stringify(response, null, 2));
            }
        });
    }
}

function buildDeleteModelCatalogCommand(commander) {
    commander
        .command('delete-model-catalog')
        .usage('<-c|--catalog-id <catalog-id>> [-p|--profile <profile>] [--debug]')
        .description('delete an existing catalog.')
        .option('-c, --catalog-id <catalog-id>', 'catalog identifier')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        if (!vendorId) {
            console.error('[Error]: cannot find the vendor-id associated with the current profile.');
            process.exit(1);
        }
        
        try {
            optionValidator.validateModelCatalogId(options.catalogId);
        } catch (err) {
            console.error(`[Error]: ${err}`);
            process.exit(1);
        }
        
        apiWrapper.callDeleteModelCatalog(options.catalogId, profile, options.debug, (data) => {
            console.log('Model catalog deleted successfully.');
        });
    }
}

function buildDeleteModelCatalogVersionCommand(commander) {
    commander
        .command('delete-model-catalog-version')
        .usage('<-c|--catalog-id <catalog-id>> <--catalog-version <catalog-version>> [-p|--profile <profile>] [--debug]')
        .description('get an existing catalog.')
        .option('-c, --catalog-id <catalog-id>', 'catalog identifier')
        .option('--catalog-version <catalog-version>', 'catalog version')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        if (!vendorId) {
            console.error('[Error]: cannot find the vendor-id associated with the current profile.');
            process.exit(1);
        }
        
        try {
            optionValidator.validateModelCatalogId(options.catalogId);
            optionValidator.validateCatalogVersion(options.catalogVersion);
        } catch (err) {
            console.error(`[Error]: ${err}`);
            process.exit(1);
        }
        
        apiWrapper.callDeleteModelCatalogVersion(options.catalogId, options.catalogVersion, profile, options.debug, (data) => {
            console.log('Model catalog version deleted successfully.');
        });
    }
}

function removeLinksFromResponse(response) {
    return R.omit(['_links'], response);
}