'use strict';

const apiWrapper = require('./api-wrapper');
const inquirer = require('inquirer');
const profileHelper = require('../utils/profile-helper');
const tools = require('../utils/tools');

// Public
module.exports = {
    createCommand: (commander) => {
        buildSubmitCommand(commander);
        buildWithdrawCommand(commander);
        buildListCertificationsCommand(commander);
        buildGetCertificationCommand(commander);
    }
};

// Private
function buildSubmitCommand(commander) {
    commander
        .command('submit')
        .usage('<-s|--skill-id <skill-id>> [-p|--profile <profile>] [--debug]')
        .option('-s, --skill-id <skill-id>', 'skill-id for the skill')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .description('submit a skill for certification')
        .action(handle);

    function handle(options) {
        if (!options.skillId) {
            console.warn('Please input required parameter: skill-id.');
            return;
        }
        let profile = profileHelper.runtimeProfile(options.profile);
        apiWrapper.callSubmit(options.skillId, profile, options.debug, () => {
            console.log('Skill submitted successfully.');
        });
    }
}

function buildWithdrawCommand(commander) {
    commander
        .command('withdraw')
        .usage('<-s|--skill-id <skill-id>> [-p|--profile <profile>] [--debug]')
        .option('-s, --skill-id <skill-id>', 'skill-id for the skill')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .description('withdraw a skill from the certification process')
        .action(handle);

    function handle(options) {
        if (!options.skillId) {
            console.warn('Please input required parameter: skill-id.');
            return;
        }
        let profile = profileHelper.runtimeProfile(options.profile);
        collectWithdrawPayload((reason, message) => {
            apiWrapper.callWithdraw(options.skillId, reason, message, profile, options.debug, () => {
                console.log('Skill withdrawn successfully.');
            });
        });
    }
}

function buildListCertificationsCommand(commander) {
    commander
    .command('list-certifications')
    .usage('<-s|--skill-id <skill-id>> [--max-results <max-results>] [--next-token <next-token>] [-p|--profile <profile>] [--debug]')
    .description('list of all certifications available for a skill')
    .option('-s, --skill-id <skill-id>', 'skill-id for the skill')
    .option('--max-results <max-results>', 'max results returned by the request')
    .option('--next-token <next-token>', 'next token if the result is paginated')
    .option('-p, --profile <profile>', 'ask cli profile')
    .option('--debug', 'ask cli debug mode')
    .action(handle);

    function handle(options) {

        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }

        if (!options.skillId) {
            console.warn('Please input required parameter: skill-id');
            return;
        }

        let profile = profileHelper.runtimeProfile(options.profile);
        apiWrapper.callListCertifications(options.skillId, options.nextToken, options.maxResults, profile, options.debug, (data) => {
            let response = tools.convertDataToJsonObject(data.body);
            if (response) {
                console.log(JSON.stringify(response, null, 2));
            }
        });
    }
}

function buildGetCertificationCommand(commander) {
    commander
    .command('get-certification')
    .usage('<-s|--skill-id <skill-id>> <-c|--certification-id <certification-id>> [--accept-language <accept-language>] [-p|--profile <profile>] [--debug]')
    .description('get certification by certification-id for a skill')
    .option('-s, --skill-id <skill-id>', 'skill-id for the skill')
    .option('-c, --certification-id <certification-id>', 'id of the certification, "~mostRecent" can be used for most recent certification')
    .option('--accept-language <accept-language>', 'preferred language variant default: en-US ')
    .option('-p, --profile <profile>', 'ask cli profile')
    .option('--debug', 'ask cli debug mode')
    .action(handle);

    function handle(options) {

        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }

        if (!options.skillId) {
            console.warn('Please input required parameter: skill-id');
            return;
        }

        if (!options.certificationId) {
            console.warn('Please input required parameter: certification-id');
            return;
        }

        let profile = profileHelper.runtimeProfile(options.profile);

        apiWrapper.callGetCertification(options.skillId, options.certificationId,options.acceptLanguage, profile, options.debug, (data) => {
            let response = tools.convertDataToJsonObject(data.body);
            if (response) {
                console.log(JSON.stringify(response, null, 2));
            }
        });
    }
}

function collectWithdrawPayload(callback) {
    const REASON_CHOICES = {
        'This is a test skill and not meant for certification': 'TEST_SKILL',
        'I want to add more features to the skill': 'MORE_FEATURES',
        'I discovered an issue with the skill': 'DISCOVERED_ISSUE',
        "I haven't received certification feedback yet": 'NOT_RECEIVED_CERTIFICATION_FEEDBACK',
        'I do not intend to publish the skill right away': 'NOT_INTEND_TO_PUBLISH',
        'Other reason': 'OTHER'
    };
    let question1 = [
        {
            type: 'list',
            name: 'reason',
            message: 'Please choose your reason for the withdrawal: ',
            choices: Object.keys(REASON_CHOICES)
        }
    ];
    inquirer.prompt(question1).then((answers) => {
        let reasonEnum = REASON_CHOICES[answers.reason];
        if (reasonEnum !== 'OTHER') {
            callback(reasonEnum, null);
        } else {
            let question2 = [
                {
                    type: 'input',
                    name: 'message',
                    message: 'Your reason: '
                }
            ];
            inquirer.prompt(question2).then((answers) => {
                callback(reasonEnum, answers.message);
            });
        }
    });
}
