const CONSTANTS = require('@src/utils/constants');
const captchaValidator = require('./captcha-validator');

module.exports = {
    validateUserQualification,
    checkPermission
};


const PermissionEnum = {
    NEW_SKILL: 'newSkill'
};
/**
 * Manage check permission and captcha validation for first time user
 *
 * @param options
 * @param vendorId
 * @param hostedSkillController
 * @param callback (error)
 * @return callback with error
 */
function validateUserQualification(options, vendorId, hostedSkillController, callback) {
    checkPermission(options, hostedSkillController, false, (error, actionUrl) => {
        if (error) {
            return callback(error);
        }

        if (!actionUrl) {
            return callback();
        }

        captchaValidator.solveCaptcha(options.browser, vendorId, actionUrl, (captchaValidateError) => {
            if (captchaValidateError) {
                return callback(captchaValidateError);
            }
            // confirm captcha has been solved
            checkPermission(options, hostedSkillController, true, (reCheckPermissionError) => {
                if (reCheckPermissionError) {
                    return callback(reCheckPermissionError);
                }
                console.log('CAPTCHA validation was successfully completed. You are able to create a Alexa hosted skill.');
                return callback();
            });
        });
    });
}

/**
 * Check if the current vendor has the permission to create a new hosted skill.
 *
 * @param options
 * @param hostedSkillController
 * @param isReCheck
 * @param callback (error, action url)
 * @return error or action url if first time to create hosted skill
 */
function checkPermission(options, hostedSkillController, isReCheck, callback) {
    hostedSkillController.getHostedSkillPermission(PermissionEnum.NEW_SKILL, options.debug, (getHostedSkillPermissionError, response) => {
        if (getHostedSkillPermissionError) {
            callback(getHostedSkillPermissionError);
            return;
        }

        if (!response) {
            callback('[Fatal]: Response from getHostedSkillPermission is not valid.');
        } else if (response.status === CONSTANTS.HOSTED_SKILL.PERMISSION_CHECK_RESULT.NEW_USER_REGISTRATION_REQUIRED) {
            if (isReCheck) {
                callback('[Error]: Failed to validate CAPTCHA. Please try again.');
            } else {
                console.log('[Info]: CAPTCHA validation is required for a new hosted skill user.');
                callback(null, response.actionUrl);
            }
        } else if (response.status === CONSTANTS.HOSTED_SKILL.PERMISSION_CHECK_RESULT.RESOURCE_LIMIT_EXCEEDED) {
            callback('[Error]: Your account has reached the maximum limit for Alexa hosted skill.\n'
                + 'Before you can create another one you must delete a hosted skill from following url: \n'
                + 'https://developer.amazon.com/alexa/console/ask');
        } else if (response.status === CONSTANTS.HOSTED_SKILL.PERMISSION_CHECK_RESULT.RATE_EXCEEDED) {
            callback('[Error]: Your hosted skills account is limited to a certain number of new hosted skills per minute. '
                + 'Please try again later.');
        } else if (response.status === CONSTANTS.HOSTED_SKILL.PERMISSION_CHECK_RESULT.ALLOWED) {
            callback();
        }
    });
}
