const retry = require('retry');
const R = require('ramda');

const apiWrapper = require('@src/api/api-wrapper');
const CONSTANTS = require('@src/utils/constants');
const tools = require('@src/utils/tools');
const profileHelper = require('@src/utils/profile-helper');

module.exports = class HostedSkillController {
    /**
     * Constructor for HostedSkillController
     */
    constructor(configuration) {
        const { profile } = configuration;
        this.profile = profile;
    }

    /**
     * Call SMAPI to create hosted skill.
     *
     * @param input
     * @param doDebug
     * @param callback (error, skillId)
     * @return callback with error or skillId
     */
    createSkill(input, doDebug, callback) {
        const vendorId = profileHelper.resolveVendorId(this.profile);
        if (!vendorId) {
            callback('[Error]: cannot find vendor id associated with the current profile.', null);
            return;
        }

        input.vendorId = vendorId;
        apiWrapper.callCreateHostedSkill(input, this.profile, doDebug, (data) => {
            const skillData = tools.convertDataToJsonObject(data.body);

            if (skillData) {
                callback(null, skillData.skillId);
            } else {
                callback('[Error]: Cannot parse the response from the create skill request. '
                    + 'Please try again with --debug option to see more details.', null);
            }
        });
    }

    /**
     * Check skill status with retry strategy,
     * callback with the final skill status after max retry times.
     *
     * @param skillId
     * @param doDebug
     * @param callback
     * @return skill provisioning status
     */
    pollingSkillStatus(skillId, doDebug, callback) {
        // The maximum waiting time:
        //   Math.min(minTimeout * Math.pow(1.2, 30), maxTimeout) ~= 8min
        // which is the same with create self hosted skill.
        const operation = retry.operation({
            retries: CONSTANTS.CONFIGURATION.RETRY.GET_PROVISIONING_STATUS.MAX_RETRY,
            minTimeout: CONSTANTS.CONFIGURATION.RETRY.GET_PROVISIONING_STATUS.MIN_TIME_OUT,
            factor: CONSTANTS.CONFIGURATION.RETRY.GET_PROVISIONING_STATUS.FACTOR
        });

        const RESOURCE_LIST = [CONSTANTS.SKILL.RESOURCES.MANIFEST,
            CONSTANTS.SKILL.RESOURCES.HOSTED_SKILL_PROVISIONING,
            CONSTANTS.SKILL.RESOURCES.INTERACTION_MODEL];

        let isRetryFinished = false;
        const response = {
            manifest: {},
            interactionModel: {},
            provisioning: {},
        };

        operation.attempt(() => {
            apiWrapper.callGetSkillStatus(skillId, RESOURCE_LIST, this.profile, doDebug, (data) => {
                if (operation.retry(isRetryFinished ? null : 'needRetry')) {
                    if (data.statusCode === 404) {
                        callback('[Error]: Skill not found. Please try again.', null);
                    } else {
                        const skillStatus = tools.convertDataToJsonObject(data.body);
                        let buildDetails;

                        response.manifest.status = R.view(R.lensPath([CONSTANTS.SKILL.RESOURCES.MANIFEST,
                            'lastUpdateRequest', 'status']), skillStatus);
                        response.manifest.errors = R.view(R.lensPath([CONSTANTS.SKILL.RESOURCES.MANIFEST,
                            'lastUpdateRequest', 'errors']), skillStatus);

                        if (skillStatus[CONSTANTS.SKILL.RESOURCES.INTERACTION_MODEL]) {
                            const locale = Object.keys(skillStatus[CONSTANTS.SKILL.RESOURCES.INTERACTION_MODEL])[0];
                            response.interactionModel.status = R.view(R.lensPath([CONSTANTS.SKILL.RESOURCES.INTERACTION_MODEL,
                                locale, 'lastUpdateRequest', 'status']), skillStatus);
                            response.interactionModel.errors = R.view(R.lensPath([CONSTANTS.SKILL.RESOURCES.INTERACTION_MODEL,
                                locale, 'lastUpdateRequest', 'errors']), skillStatus);
                            buildDetails = R.view(R.lensPath([CONSTANTS.SKILL.RESOURCES.INTERACTION_MODEL, locale,
                                'lastUpdateRequest', 'buildDetails']), skillStatus);
                        }

                        response.provisioning.status = R.view(R.lensPath([CONSTANTS.SKILL.RESOURCES.HOSTED_SKILL_PROVISIONING,
                            'lastUpdateRequest', 'status']), skillStatus);

                        if (!response.manifest.status && !response.provisioning.status && !response.interactionModel.status) {
                            callback(null, response);
                            isRetryFinished = true;
                        }

                        if (response.manifest.status === CONSTANTS.SKILL.MANIFEST_STATUS.FAILURE
                            || response.provisioning.status === CONSTANTS.SKILL.PROVISIONING_STATUS.FAILURE
                            || (response.interactionModel.status === CONSTANTS.SKILL.INTERACTION_MODEL_STATUS.FAILURE && buildDetails)
                            || (response.provisioning.status === CONSTANTS.SKILL.PROVISIONING_STATUS.SUCCESS
                                && response.manifest.status === CONSTANTS.SKILL.MANIFEST_STATUS.SUCCESS
                                && response.interactionModel.status === CONSTANTS.SKILL.INTERACTION_MODEL_STATUS.SUCCESS)) {
                            callback(null, response);
                            isRetryFinished = true;
                        }
                    }
                }

                if (operation.attempts() === CONSTANTS.CONFIGURATION.RETRY.GET_PROVISIONING_STATUS.MAX_RETRY) {
                    callback(null, response);
                    isRetryFinished = true;
                }
            });
        });
    }

    /**
     * Call SMAPI to check if current user is allowed to invoke HostedSkill functions.
     *
     * @param permissionType enum with newSkill
     * @param doDebug
     * @param callback (error, permissionInfo)
     * @return callback with error or permissionInfo
     */
    getHostedSkillPermission(permissionType, doDebug, callback) {
        const vendorId = profileHelper.resolveVendorId(this.profile);
        if (!vendorId) {
            callback('[Error]: cannot find vendor id associated with the current profile.');
            return;
        }

        apiWrapper.getHostedSkillPermission(vendorId, permissionType, this.profile, doDebug, (data) => {
            const permissionInfo = tools.convertDataToJsonObject(data.body);
            if (permissionInfo) {
                callback(null, permissionInfo);
            } else {
                callback('[Error]: Cannot parse the response from the getHostedSkillPermission request. '
                    + 'Please try again with --debug option to see more details.');
            }
        });
    }
};
