const inquirer = require('inquirer');
const gitClient = require('../clients/git-client');
const alexaHostedUtils = require('../utils/alexaHostedSkill-utils');
const profileHelper = require('../utils/profile-helper');
const displayUtil = require('../utils/display-utility');

module.exports = {
    checkProfileBeforeDeploying: checkProfileBeforeDeploying,
    checkStateBeforeDeploying: checkStateBeforeDeploying,
    deployHostedSkill: deployHostedSkill
}

/**
 * Deploys an Alexa Hosted Skill
 */
function deployHostedSkill(projectPath, doDebug, force) {
    const verbosityOptions = {
        showCommand: doDebug ? true : false,
        showOutput: doDebug ? true : false
    }

    const spinner = new displayUtil.Spinner('Deploying Alexa Hosted Skill...');
    if (!doDebug) {
        spinner.start();
    }

    if (doDebug) {
        console.log(`- Checking for uncommited changes in dev branch before deploying...`);
    }
    if (gitClient.isBranchOneAheadOfBranchTwo(projectPath, 'master', 'dev')) {
        console.warn(`[Warn]: Some commits on master branch are not in dev branch`);
    }
    if (doDebug) {
        console.log(`- Merging dev branch into master branch...`);
    }
    gitClient.mergeBranchOneIntoBranchTwo(projectPath, 'dev', 'master', verbosityOptions);
    if (doDebug) {
        console.log(`- Pushing dev branch...`);
    }
    gitClient.push(projectPath, 'dev', verbosityOptions, force);
    if (doDebug) {
        console.log(`- Pushing master branch...`);
    }
    gitClient.push(projectPath, 'master', verbosityOptions, force);
    if (doDebug) {
        console.log();
    }

    if (!doDebug) {
        spinner.stop();
    }
}

/**
 * Checks if the specified profile matches the original profile used to clone the alexa hosted skill
 * @param projectPath
 * @param profile
 * @param cb
 */
function checkProfileBeforeDeploying(projectPath, profile, cb) {
    const ahsProfile = alexaHostedUtils.getAlexaHostedSkillProfile(projectPath);
    // Ensure backward compatibility
    if (!ahsProfile) {
        console.log('We notice you haven\'t linked your ask-cli profile to current hosted skill.')
        // Get all profiles
        const profiles = profileHelper.getListProfile().map(p => p.askProfile);
        // inquirer select original profile and write it
        const profileChoiceQuestion = {
            type: 'list',
            message: 'Please select the profile that you\'d like to link with:',
            name: 'profile',
            choices: profiles,
        };
        inquirer.prompt([profileChoiceQuestion]).then((answer) => {
            const originalProfile = answer.profile;
            alexaHostedUtils.addAlexaHostedSkillProfileToConfig(projectPath, originalProfile);
            cb();
        });
    } else if (ahsProfile !== profile) {
        cb(`[Warn]: Please use the original profile that was specified when cloning your skill. (${ahsProfile})`);
    } else {
        cb();
    }
}

/**
 * Checks if current branch is dev and if the state is not dirty
 * @param {String} projectPath
 */
function checkStateBeforeDeploying(projectPath, cb) {
    if (gitClient.getCurrentBranch(projectPath) !== 'dev') {
        cb(`[Warn]: Please switch to "dev" branch and make sure your latest changes are there.\nTo switch, run the following command: git checkout dev`);
        return;
    }

    if (gitClient.isBranchStateDirty(projectPath)) {
        const confirm = {
            type: 'confirm',
            name: 'isAllowed',
            message: 'You have uncommited changes in your dev branch. Your skill code changes will not be deployed if you don\'t commit them. Do you still want to deploy your skill?',
        };
        inquirer.prompt(confirm).then((answer) => {
            if (!answer.isAllowed) {
                cb(null, false);
                return;
            }
            cb(null, true);
        });
    } else {
        cb(null, true);
    }
}
