const jsonfile = require('jsonfile');
const async = require('async');
const path = require('path');

const urlUtils = require('@src/utils/url-utils');
const strUtils = require('@src/utils/string-utils');
const lambdaEndpointUtility = require('@src/utils/lambda-operation/lambda-endpoint-utility');
const parser = require('../utils/skill-parser');
const profileHelper = require('../utils/profile-helper');
const jsonUtility = require('../utils/json-utility');
const domainRegistry = require('../utils/domain-registry');
const retrieveLambda = require('../utils/lambda-operation/retrieve-lambda');
const tools = require('../utils/tools');
const alexaHostedSkillUtils = require('../utils/alexaHostedSkill-utils');

// Public
module.exports = {
    updateSkillSchemaWithLambdaCreation,
    parseSkill
};

function updateSkillSchemaWithLambdaCreation(generatedLambdaList, submittingReadySkillSchema) {
    const finalSkillSchema = JSON.parse(JSON.stringify(submittingReadySkillSchema));
    if (!generatedLambdaList || generatedLambdaList.length === 0) {
        return finalSkillSchema;
    }
    for (const metaData of generatedLambdaList) {
        for (const usage of metaData.alexaUsage) {
            const domain = domainRegistry.getSkillSchemaKey(usage.split('/')[0]);
            const region = usage.split('/')[1];
            const arrayPathToUri = ['manifest', 'apis', domain, 'regions', region, 'endpoint', 'uri'];
            jsonUtility.insertObjectToObject(finalSkillSchema, arrayPathToUri, metaData.arn);
        }
    }
    // Logic to set one region as default.
    for (const domain of Object.keys(finalSkillSchema.manifest.apis)) {
        const domainInfo = finalSkillSchema.manifest.apis[domain];
        if (domainInfo.endpoint) {
            // If default region existed and need to create Lambda, replace it with real ARN
            if (domainInfo.regions.default) {
                domainInfo.endpoint.uri = domainInfo.regions.default.endpoint.uri;
                jsonUtility.deletePropertyFromJsonObject(domainInfo, ['regions', 'default']);
            }
        } else {
            // If NA exists move NA up; else move whatever comes the first
            const regionsKeyArray = Object.keys(domainInfo.regions);
            if (regionsKeyArray.indexOf('NA') === -1) {
                const firstRegion = domainInfo.regions[regionsKeyArray[0]];
                const defaultEndpoint = firstRegion.endpoint;
                domainInfo.endpoint = defaultEndpoint;
                jsonUtility.deletePropertyFromJsonObject(domainInfo, ['regions', regionsKeyArray[0]]);
            } else {
                const defaultEndpoint = domainInfo.regions.NA;
                domainInfo.endpoint = defaultEndpoint;
                jsonUtility.deletePropertyFromJsonObject(domainInfo, ['regions', 'NA']);
            }
        }
        // Clean empty regions if no endpoint inside
        if (Object.keys(domainInfo.regions) === 0) {
            jsonUtility.deletePropertyFromJsonObject(domainInfo, ['regions']);
        }
    }
    return finalSkillSchema;
}

function parseSkill(projectConfigFile, skillFile, profile, callback) {
    const manifest = jsonUtility.read(skillFile);
    if (!isValidManifest(manifest)) {
        console.error('[Error]: Invalid skill manifest. Make sure your skill.json always contains "manifest" field.');
        return;
    }

    const askConfig = jsonUtility.read(projectConfigFile);
    if (!askConfig.deploy_settings[profile] || Object.keys(askConfig.deploy_settings[profile]).length === 0) {
        addNewProfileToConfig(askConfig, profile);
        jsonfile.writeFileSync(projectConfigFile, askConfig, { spaces: 2 });
    }

    const skillId = jsonUtility.getPropertyValueFromObject(askConfig, ['deploy_settings', profile, 'skill_id']);

    // 1.merge skill.json and the "merge" field from .ask/config
    const mergedManifest = tools.collate(manifest, askConfig.deploy_settings[profile].merge);
    // move default endpoint under regions to treat it as regular endpoint
    moveDefaultEndpointIntoRegions(mergedManifest);

    // 2.validate user's Lambda settings, and form a pre-processed manifest.
    // this Manifest has sourceDir and url (either true ARN when the functionName is found, or the functionName that will be created)
    generateSubmittingReadyManifest(mergedManifest, profile, (preprocessedManifest) => {
        // 3.extract skillInfo data object
        const withTrueURLManifest = tools.collate(mergedManifest, preprocessedManifest);
        moveDefaultEndpointUpBack(withTrueURLManifest);
        const skillInfo = parser.extractSkillInfo(withTrueURLManifest);
        // 4.move default endpoint back and finish the parsing
        moveDefaultEndpointUpBack(preprocessedManifest);
        callback(preprocessedManifest, skillInfo, skillId);
    });
}

function moveDefaultEndpointIntoRegions(mergedManifest) {
    const apisInfo = jsonUtility.getPropertyValueFromObject(mergedManifest, ['manifest', 'apis']);
    for (const domain of Object.getOwnPropertyNames(apisInfo)) {
        const domainInfo = apisInfo[domain];
        if (domainInfo.endpoint) {
            if (!domainInfo.regions) {
                domainInfo.regions = {};
            }
            domainInfo.regions.default = {
                endpoint: domainInfo.endpoint
            };
        }
    }
    return mergedManifest;
}

function generateSubmittingReadyManifest(mergedManifest, profile, callback) {
    const deepCopiedManifest = JSON.parse(JSON.stringify(mergedManifest));
    const awsProfile = profileHelper.getAWSProfile(profile);
    const isAlexaHostedSkill = alexaHostedSkillUtils.isAlexaHostedSkill(process.cwd(), profile);
    async.each(domainRegistry.domainList(), (domain, domainAsyncCallback) => {
        if (isAlexaHostedSkill) {
            domainAsyncCallback();
            return;
        }
        const domainKey = domainRegistry.getSkillSchemaKey(domain);
        const pathToRegions = ['manifest', 'apis', domainKey, 'regions'];
        const regionsInfo = jsonUtility.getPropertyValueFromObject(deepCopiedManifest, pathToRegions);
        if (!regionsInfo || Object.getOwnPropertyNames(regionsInfo).length === 0) {
            domainAsyncCallback();
            return;
        }
        async.each(Object.getOwnPropertyNames(regionsInfo), (region, regionLevelCallback) => {
            const currentUri = regionsInfo[region].endpoint.uri;
            const currentSourceDir = regionsInfo[region].endpoint.sourceDir;
            const alexaUsage = `${domain}/${region}`;

            // 1.Validate current uri (after merge) exists
            if (typeof currentUri !== 'string' || currentUri.trim().length === 0) {
                regionLevelCallback(`[Error]: Please provide the uri (either ARN or function name if using Lambda) for ${alexaUsage}.`);
                return;
            }

            // 2.uri is Lambda ARN or Lambda function name
            if (urlUtils.isLambdaArn(currentUri) || strUtils.isLambdaFunctionName(currentUri)) {
                if (!awsProfile) {
                    if (typeof currentSourceDir === 'string' && currentSourceDir.trim().length !== 0) {
                        // when user doesn't set awsProfile but still provides sourceDir, delete the region and warn out
                        console.warn(`[Warn]: CLI could not deploy the Lambda when AWS profile not found from current profile [${profile}]. \
Delete the Lambda endpoint for ${alexaUsage} to unblock the deploy process.`);
                        delete regionsInfo[region];
                    }
                    regionLevelCallback();
                    return;
                }
                // 2.1 uri is Lambda Arn
                if (urlUtils.isLambdaArn(currentUri)) {
                    const awsRegion = lambdaEndpointUtility.resolveLambdaAwsRegion(currentUri, region, awsProfile);
                    retrieveLambda.retrieveLambdaFunction(currentUri, awsRegion, awsProfile, (err, res) => {
                        if (err) {
                            regionLevelCallback(`[Error]: Get Lambda ARN ${currentUri} failure for ${alexaUsage}.\n${err}`);
                            return;
                        }
                        if (!res) {
                            regionLevelCallback(`[Error]: The Lambda ARN ${currentUri} doesn't exist for ${alexaUsage}.`);
                            return;
                        }
                        regionLevelCallback();
                    });
                } else { // 2.2 uri is Lambda function name
                    const awsRegion = lambdaEndpointUtility.resolveLambdaAwsRegion(null, region, awsProfile);
                    // validate if users setting for Lambda endpoint is correct
                    try {
                        validateEndpointSettingsOverExistingResources(domain, region, awsRegion, currentUri, profile);
                    } catch (validateDefaultErr) {
                        regionLevelCallback(validateDefaultErr);
                        return;
                    }
                    // retrieve Lambda based on the function name
                    retrieveLambda.retrieveLambdaFunction(currentUri, awsRegion, awsProfile, (err, res) => {
                        if (err) {
                            regionLevelCallback(`[Error]: Get Lambda ARN ${currentUri} failure for ${alexaUsage}.\n${err}`);
                            return;
                        }
                        if (!res) {
                            if (typeof currentSourceDir === 'string' && currentSourceDir.trim().length !== 0) {
                                // this is a valid case if Lambda function does not exist
                                regionLevelCallback();
                            } else {
                                regionLevelCallback(`[Error]: Failed to resolve Lambda endpoint for ${alexaUsage} without sourceDir found.
Please specify sourceDir if you want to create Lambda with the function name, or provide the Lambda ARN alone if you want to set the endpoint only.`);
                            }
                        } else {
                            // if Lambda function exists, replace it with true Lambda ARN
                            const awsResponseURL = res.Configuration.FunctionArn;
                            if (urlUtils.isLambdaArn(awsResponseURL)) {
                                const arrayPathToUri = [region, 'endpoint', 'uri'];
                                jsonUtility.insertObjectToObject(regionsInfo, arrayPathToUri, awsResponseURL);
                                regionLevelCallback();
                            } else {
                                regionLevelCallback(`[Error]: Please make sure you use correct Lambda function name for ${alexaUsage}.`);
                            }
                        }
                    });
                }
            } else {
                // 3.uri is https endpoint
                // if uri is not Lambda Arn nor Lambda function name,
                // it will be considered as https url
                if (typeof currentSourceDir === 'string' && currentSourceDir.trim().length !== 0) {
                    regionLevelCallback(`[Error]: CLI doesn't support sourceDir deploy when the endpoint of ${alexaUsage} is not Lambda.`);
                } else {
                    regionLevelCallback();
                }
            }
        }, (err) => {
            if (err) {
                domainAsyncCallback(err);
            }
            domainAsyncCallback();
        });
    }, (err) => {
        if (err) {
            console.error(err);
            process.exit(1);
        }
        obtainUniqueness(deepCopiedManifest, (approvedManifest) => {
            callback(approvedManifest);
        });
    });
}

// check whether multiple different code bases map to a single lambda uri, and remove 'sourceDir' and pass to callback.
function obtainUniqueness(manifest, callback) {
    const checkUniqueMap = new Map();
    const domainKeyArray = domainRegistry.domainList().map(domain => domainRegistry.getSkillSchemaKey(domain));

    for (const domainKey of domainKeyArray) {
        if (!manifest.manifest.apis[domainKey] || !manifest.manifest.apis[domainKey].regions) {
            continue;
        }
        const regionsInfo = manifest.manifest.apis[domainKey].regions;
        for (const region of Object.getOwnPropertyNames(regionsInfo)) {
            const { uri } = regionsInfo[region].endpoint;
            if (!uri) {
                continue;
            }
            const deletePath = ['manifest', 'apis', domainKey, 'regions', region, 'endpoint', 'sourceDir'];
            if (uri.substr(0, 4) !== 'arn:') {
                // it's a function name, which aws don't have a function match with
                jsonUtility.deletePropertyFromJsonObject(manifest, deletePath);
                continue;
            }
            const { sourceDir } = regionsInfo[region].endpoint;
            if (!sourceDir || sourceDir.length === 0) {
                continue;
            }
            if (!checkUniqueMap.get(uri)) {
                checkUniqueMap.set(uri, sourceDir);
            } else if (sourceDir !== checkUniqueMap.get(uri)) {
                console.error(`[Error]: Multiple sourceDir pointing to the same endpoint ${uri}.`);
                process.exit(1);
            }
            jsonUtility.deletePropertyFromJsonObject(manifest, deletePath);
        }
    }
    callback(manifest);
}

function moveDefaultEndpointUpBack(manifest) {
    const apisInfo = jsonUtility.getPropertyValueFromObject(manifest, ['manifest', 'apis']);
    for (const domain of Object.getOwnPropertyNames(apisInfo)) {
        if (apisInfo[domain].regions) {
            apisInfo[domain].endpoint = jsonUtility.getPropertyValueFromObject(
                apisInfo, [domain, 'regions', 'default', 'endpoint']
            );
            jsonUtility.deletePropertyFromJsonObject(apisInfo, [domain, 'regions', 'default']);
            // Remove "regions" field if there is no region inside
            if (Object.keys(apisInfo[domain].regions).length === 0) {
                jsonUtility.deletePropertyFromJsonObject(apisInfo, [domain, 'regions']);
            }
        }
    }
}

/**
 * Verify if user's setting from skill.json is consistent in resources config for Lambda functions
 * @param {String} domain Alexa domain for the Lambda
 * @param {String} region Alexa region for the Lambda
 * @param {String} awsRegion AWS region value from current user setting
 * @param {String} functionName function name of the Lambda from current user setting
 * @param {String} profile
 */
function validateEndpointSettingsOverExistingResources(domain, region, awsRegion, functionName, profile) {
    const INVALID_LAMBDA_RESOURCE_ERROR = 'Invalid Lambda resources config in ./ask/config.';
    const alexaUsage = `${domain}/${region}`;
    const lambdaResourcesList = jsonUtility.getPropertyValueFromObject(
        jsonUtility.read(path.join(process.cwd(), '.ask', 'config')), ['deploy_settings', profile, 'resources', 'lambda']
    ) || [];
    for (const lambdaResource of lambdaResourcesList) {
        if (lambdaResource.alexaUsage && lambdaResource.alexaUsage.includes(alexaUsage) && strUtils.isNonBlankString(lambdaResource.arn)) {
            if (urlUtils.isLambdaArn(lambdaResource.arn)) {
                const existingLambda = lambdaEndpointUtility.parseLambdaArn(lambdaResource.arn);
                // functionName is same but awsRegion is not
                if (functionName === existingLambda.functionName && existingLambda.awsRegion !== awsRegion) {
                    const LAMBDA_REGION_DOC_LINK = 'https://developer.amazon.com/docs/smapi/ask-cli-intro.html#lambda-region';
                    throw `[Error]: ${INVALID_LAMBDA_RESOURCE_ERROR} The current awsRegion for ${alexaUsage} is ${awsRegion}, \
which is not same as the existing Lambda function ${lambdaResource.arn} in .ask/config.
Please make sure you set the awsRegion correctly for the default region's Lambda function by either setting AWS_REGION environmental variable \
or modify the "region" value under your AWS profile in ~/.aws/config file.
For more details, please check ${LAMBDA_REGION_DOC_LINK}.`;
                } else if (functionName !== existingLambda.functionName) {
                    // functionName is not same
                    throw `[Error]: ${INVALID_LAMBDA_RESOURCE_ERROR} \
Failed to replace the Lambda function for ${alexaUsage} with ${functionName} in AWS region ${awsRegion}.
Please first delete the Lambda resource with alexaUsage "${alexaUsage}" from .ask/config if you need to replace the Lambda linkage for this skill.`;
                }
            } else {
                // Lambda arn is invalid
                throw `[Error]: ${INVALID_LAMBDA_RESOURCE_ERROR} \
Lambda ARN is invalid for "${alexaUsage}", please correct the Lambda ARN or delete this Lambda resource.`;
            }
            break;
        }
    }
}

function addNewProfileToConfig(askConfig, profile) {
    askConfig.deploy_settings[profile] = {
        skill_id: '',
        was_cloned: false,
        merge: {}
    };
}

// a basic validation for skill.json
// to check whether it has manifest or not
function isValidManifest(skillSchema) {
    return skillSchema && skillSchema.manifest && typeof skillSchema.manifest === 'object';
}
