const fs = require('fs');
const path = require('path');
const readline = require('readline');
const { spawn } = require('child_process');
const os = require('os');
const jsonUtility = require('../utils/json-utility');
const tools = require('../utils/tools');
const jsonfile = require('jsonfile');
const alexaHostedSkillUtils = require('../utils/alexaHostedSkill-utils');
const CONSTANTS = require('../utils/constants');

const apiWrapper = require('../api/api-wrapper');

module.exports = {
    createCommand: (commander) => {
        commander
            .command('git-credentials-helper')
            .description('gets temporary git credentials for hosted skill repo')
            .option('-o, --operation <operation>', 'if not defined, will get operation')
            .option('-p, --profile <profile>', 'ask cli profile')
            .option('-h, --help', 'output usage information', () => {
                console.log(CONSTANTS.COMMAND.CREDENTIALS.HELP_DESCRIPTION);
                process.exit(0);
            })
            .action(handle);

        function handle(options) {
            
            const currentDir = process.cwd();
            const profile = options.profile || 'default';
            const operation = options.operation || 'get';
            const isHostedSkill = alexaHostedSkillUtils.isAlexaHostedSkill(currentDir, profile);

            if (!isHostedSkill) {
                console.error('[Error]: This folder does not contain an Alexa Hosted Skill');
                return;
            }

            // Gets git credentials
            if (operation === 'get') {

                getHostInfoFromStdIn((hostInfo) => {

                    // Checks if cached credentials exist and if they are not expired
                    const cachedCredentials = getCredentialsFromCache(currentDir, profile);
                    if (cachedCredentials && new Date(cachedCredentials.expiresAt).getTime() > new Date().getTime()) {
                        console.log(`username=${cachedCredentials.username}`);
                        console.log(`password=${cachedCredentials.password}`);
                        return;
                    }

                    const skillId = alexaHostedSkillUtils.getSkillId(currentDir, profile);
                    if (!skillId) {
                        console.error('[Error]: cannot find skillId in .ask/config');
                        return;
                    }

                    const gitRepoUrl = `${hostInfo.protocol}://${hostInfo.host}/${hostInfo.path}`;

                    apiWrapper.getGitCredentials(skillId, gitRepoUrl, profile, options.debug, (data) => {

                        let credentials;
                        if (data.statusCode === 200) {
                            credentials = tools.convertDataToJsonObject(data.body).repositoryCredentials;
                        } else {
                            console.error(`Call get-git-credentials error: Response status code: ${data.statusCode}`);
                            process.exit(1);
                        }

                        console.log(`username=${credentials.username}`);
                        console.log(`password=${credentials.password}`);
                    });

                });

            // Store credentials in cache
            } else if (operation === 'store') {

                const DEFAULT_CREDENTIALS_EXPIRY_TIME_IN_MINUTES = 15;
                getCredentialsFromStdIn((credentials) => {
                    
                    if (!credentials.expiresAt) {
                        credentials.expiresAt = 0;
                    }
                    cacheCredentials(credentials, currentDir);
                    deleteSystemAndGlobalCachedCredentials(credentials.protocol, credentials.host, () => {
                        process.exit(0);
                    })
                });

            // Delete credentials from cache
            } else if (operation === 'erase') {
                
                deleteCachedCredentials(currentDir, profile);

            }
        }
    }
}


/**
 * Called when git credentials helper calls the "get" function
 * @param {*} callback 
 */
function getHostInfoFromStdIn(callback) {
    // Read stdin
    const rl = readline.createInterface({
        input: process.stdin,
    });
    const hostInfo = {
        protocol: null,
        host: null,
        path: null,
    };
    let lineIndex = 0;
    rl.on('line', (line) => {
        lineIndex++;
        hostInfo.protocol = line.split('protocol=')[1] || hostInfo.protocol;
        hostInfo.host = line.split('host=')[1] || hostInfo.host;
        hostInfo.path = line.split('path=')[1] || hostInfo.path;
        if (lineIndex === 3) {
            rl.close();
            callback(hostInfo);
        }
    });
}

/**
 * Called when git credentials helper calls the "store" function
 * @param {*} callback 
 */
function getCredentialsFromStdIn(callback) {
    // Read stdin
    const rl = readline.createInterface({
        input: process.stdin,
    });
    const credentials = {
        protocol: null,
        host: null,
        path: null,
        username: null,
        password: null,
    };
    let lineIndex = 0;
    rl.on('line', (line) => {
        lineIndex++;
        credentials.protocol = line.split('protocol=')[1] || credentials.protocol;
        credentials.host = line.split('host=')[1] || credentials.host;
        credentials.path = line.split('path=')[1] || credentials.path;
        credentials.username = line.split('username=')[1] || credentials.username;
        credentials.password = line.split('password=')[1] || credentials.password;
        if (lineIndex === 5) {
            rl.close();
            callback(credentials);
        }
    });
}

/**
 * Gets credentials from cache
 */
function getCredentialsFromCache(projectPath) {
    const configFilePath = path.join(projectPath, '.ask', 'config');
    const config = jsonUtility.read(configFilePath);
    if (!(config && config.alexaHosted && config.alexaHosted.gitCredentialsCache)) {
        return null;
    }
    const credentialsObject = config.alexaHosted.gitCredentialsCache;
    return credentialsObject;
}

/**
 * Caches the temporary git credentials in .ask/.tempGitCredentials file
 * @param {Object} credentials
 */
function cacheCredentials(credentials, projectPath) {
    const configFilePath = path.join(projectPath, '.ask', 'config');
    const config = jsonUtility.read(configFilePath);
    config.alexaHosted.gitCredentialsCache = credentials;
    jsonfile.writeFileSync(configFilePath, config, { spaces: 2 })
}

function deleteCachedCredentials(projectPath) {
    const configFilePath = path.join(projectPath, '.ask', 'config');
    const config = jsonUtility.read(configFilePath);
    config.alexaHosted.gitCredentialsCache = undefined;
    jsonfile.writeFileSync(configFilePath, config, { spaces: 2 })
}

function deleteSystemAndGlobalCachedCredentials(protocol, host, callback) {
    // create temp folder
    const tmpFolder = fs.mkdtempSync(`${os.tmpdir()}${path.sep}`);
    // spawn process inside of the temp folder and execute the git command + send input
    const childProcess = spawn(`git credential reject`, {shell: true, windowsHide: true, cwd: tmpFolder});
    childProcess.stdin.write(`protocol=${protocol}\n`);
    childProcess.stdin.write(`host=${host}\n`);
    childProcess.stdin.write(`\n`); // Needs 3rd input before executing and closing
    childProcess.once('close', () => {
        // delete tmp folder
        fs.rmdirSync(tmpFolder);
        callback();
    });
}
