'use strict';
const expect = require('chai').expect;
const sinon = require('sinon');
const tools = require('@src/utils/tools');
const helper = require('@src/commands/new/helper');
const ui = require('@src/commands/new/ui');
const httpClient = require('@src/clients/http-client');
const gitClient = require('@src/clients/git-client');
const displayUtility = require('@src/utils/display-utility');
const urlUtils = require('@src/utils/url-utils');

const noop = () => {};

describe('Commands new test - helper test', () => {
    const TEST_URL = 'https://test.com';
    const TEST_ERROR = 'TestError;'

    describe('# getTemplateMap check', () => {
        beforeEach(() => {
            sinon.stub(httpClient, 'request');
            sinon.stub(tools, 'convertDataToJsonObject');
        });

        it('| httpClient request gets called successfully', () => {
            //setup
            const TEST_PARAMS = {
                url : TEST_URL,
                method : 'GET'
            };
            //call
            helper.getTemplateMap(TEST_URL, false, null);
            // verify
            expect(httpClient.request.args[0][0]).deep.equal(TEST_PARAMS);
        });
        
        it('| httpClient request returns error, expect error callback', (done) => {
            // setup
            httpClient.request.callsArgOnWith(3, {}, TEST_ERROR, null);
            // call
            helper.getTemplateMap(TEST_URL, false, (error, response) => {
                // verify
                expect(error).equal('[Error]: Cannot retrieve the template list.');
                expect(response).to.be.null;
                done();
            });
        });

        it('| httpClient request returns null status code, expect error callback ', (done) => {
            // setup
            const TEST_RESPONSE = {
                statusCode : null
            };
            httpClient.request.callsArgOnWith(3, {}, null, TEST_RESPONSE);
            // call
            helper.getTemplateMap(TEST_URL, false, (error, response) => {
                // verify
                expect(error).equal('[Error]: Cannot retrieve the template list.');
                expect(response).to.be.null;
                done();
            });
        });

        it('| httpClient request returns successfully, expect a response body', (done) => {
            // setup
            const TEST_RESPONSE = {
                statusCode: 200,
                body:'test'
            };
            tools.convertDataToJsonObject.returns(TEST_RESPONSE);
            httpClient.request.callsArgOnWith(3, {}, null, TEST_RESPONSE);
            // call
            helper.getTemplateMap(TEST_URL, false, (error, response) => {
                // verify
                expect(error).to.be.null;
                expect(response).equal(TEST_RESPONSE);
                done();
            })
        });

        afterEach(() => {
            sinon.restore();
        });
    });

    describe('# gitCloneTemplate check', () => {
        const TEST_PACKAGE_NAME = 'testPackageName';
        const TEST_ERROR = 'testError';
        let sandbox;

        beforeEach(() => {
            sandbox = sinon.createSandbox();
            sandbox.stub(ui, 'getConfirmation');
            sandbox.stub(console, 'warn');
            sandbox.stub(console, 'error');
            sandbox.stub(process, 'exit');
            sandbox.stub(urlUtils, 'isUrlOfficialTemplate');
            sandbox.stub(gitClient, 'clone');
        });

        it('| cloneUrl is not from official templates, expects console warn and user confirmation', () => {
            // setup
            urlUtils.isUrlOfficialTemplate.callsFake( () => false);
            // call
            helper.gitCloneTemplate(TEST_URL, TEST_PACKAGE_NAME, noop);
            // verify
            expect(console.warn.args[0][0]).equals('[Warn]: Downloading skill template from unofficial resource. '
            + 'Please make sure you understand what each script is doing to best protect yourself from malicious usage');
            
        });

        it('| cloneUrl user confirmation returns error, expect console error', () => {
            // setup
            urlUtils.isUrlOfficialTemplate.callsFake( () => false);
            ui.getConfirmation.callsArgOnWith(0,{}, TEST_ERROR, null);
            // call
            helper.gitCloneTemplate(TEST_URL, TEST_PACKAGE_NAME, (err) => {
                // verify
                expect(err).equal(TEST_ERROR);
            });
        });

        it('| cloneUrl user confirmation returns false, expect termination', () => {
            // setup
            urlUtils.isUrlOfficialTemplate.callsFake( () => false);
            ui.getConfirmation.callsArgOnWith(0,{}, null, false);
            // call
            helper.gitCloneTemplate(TEST_URL, TEST_PACKAGE_NAME, noop);
            // verify
            expect(process.exit.args[0][0]).equal(1);
        });

        it('| cloneUrl user confirmation returns true, git clone is called successfully', () => {
            // setup
            urlUtils.isUrlOfficialTemplate.callsFake( () => false);
            ui.getConfirmation.callsArgOnWith(0,{}, null, true);
            // call
            helper.gitCloneTemplate(TEST_URL, TEST_PACKAGE_NAME, noop);
            // verify
            expect(gitClient.clone.args[0][0]).equal(TEST_URL);
            expect(gitClient.clone.args[0][1].split('/').pop()).equal(TEST_PACKAGE_NAME);
        });
        
        it('| check if git clone is called correctly', () => {
            // setup
            urlUtils.isUrlOfficialTemplate.callsFake( () => true);
            // call
            helper.gitCloneTemplate(TEST_URL, TEST_PACKAGE_NAME, noop );
            // verify
            expect(gitClient.clone.args[0][0]).equal(TEST_URL);
            expect(gitClient.clone.args[0][1].split('/').pop()).equal(TEST_PACKAGE_NAME);
        });

        it('| git clone returns error, expect callback', () => {
            // setup
            urlUtils.isUrlOfficialTemplate.callsFake( () => true);
            gitClient.clone.callsArgOnWith(2, {}, TEST_ERROR);
            // call
            helper.gitCloneTemplate(TEST_URL, TEST_PACKAGE_NAME, (err) => {
                // verify
                expect(err).equal(TEST_ERROR);
            });
        });
        afterEach(() => {
            sandbox.restore();   
        });
    });
});