const { expect } = require('chai');
const sinon = require('sinon');

const apiWrapper = require('@src/api/api-wrapper');
const HostedSkillController = require('@src/controller/hosted-skill-controller/index');
const profileHelper = require('@src/utils/profile-helper');
const CONSTANTS = require('@src/utils/constants');

describe('Controller test - hosted skill controller test', () => {
    const TEST_SKILL_ID = 'skillId';
    const TEST_VENDOR_ID = 'vendorId';
    const TEST_PROFILE = 'testProfile';
    const TEST_RUNTIME = 'Node.js 8.10';
    const PERMISSION_TYPE = 'newSkill';
    const PERMISSION_RESULT = 'ALLOWED';

    const TEST_CONFIGURATION = {
        profile: TEST_PROFILE
    };

    const INPUT = {
        runtime: TEST_RUNTIME,
        manifest: {
            manifest: {}
        }
    };

    const SKILL_DATA = {
        body: {
            skillId: TEST_SKILL_ID
        }
    };
    const hostedSkillController = new HostedSkillController(TEST_CONFIGURATION);

    describe('# inspect correctness for constructor', () => {
        it('| initiate as a HostedSkillController class', () => {
            expect(hostedSkillController).to.be.instanceOf(HostedSkillController);
        });
    });

    describe('# createSkill method', () => {
        let sandbox;
        beforeEach(() => {
            sandbox = sinon.createSandbox();
            sandbox.stub(profileHelper, 'resolveVendorId');
            sandbox.stub(apiWrapper, 'callCreateHostedSkill');
        });

        it('| No vendorId found in profile, expect console error', (done) => {
            // set up
            profileHelper.resolveVendorId.withArgs(TEST_PROFILE).returns(null);
            const NO_VENDOR_ID_ERROR = '[Error]: cannot find vendor id associated with the current profile.';
            // call
            hostedSkillController.createSkill(INPUT, false, (error, skillId) => {
                // verify
                expect(NO_VENDOR_ID_ERROR === error);
                expect(skillId == null);
                done();
            });
        });

        it('| create skill return invalid data', (done) => {
            // set up
            profileHelper.resolveVendorId.withArgs(TEST_PROFILE).returns(TEST_VENDOR_ID);
            apiWrapper.callCreateHostedSkill.callsArgOnWith(3, {}, {});
            const CREATE_SKILL_ERROR = '[Error]: Cannot parse the response from the create skill request. '
                + 'Please try again with --debug option to see more details.';
            // call
            hostedSkillController.createSkill(INPUT, false, (error, skillId) => {
                // verify
                expect(CREATE_SKILL_ERROR === error);
                expect(skillId == null);
                done();
            });
        });

        it('| create skill return valid data', (done) => {
            // set up
            profileHelper.resolveVendorId.withArgs(TEST_PROFILE).returns(TEST_VENDOR_ID);
            apiWrapper.callCreateHostedSkill.callsArgOnWith(3, {}, SKILL_DATA);

            // call
            hostedSkillController.createSkill(INPUT, false, (error, skillId) => {
                // verify
                expect(error == null);
                expect(skillId === TEST_SKILL_ID);
                done();
            });
        });

        afterEach(() => {
            apiWrapper.callCreateHostedSkill.restore();
            sandbox.restore();
        });
    });


    describe('# pollingSkillStatus method', () => {
        const INVALID_RESPONSE_ERROR = '[Error]: Skill not found. Please try again.';

        let sandbox;
        const data = {
            statusCode: 200,
            body: {
                manifest: {
                    lastUpdateRequest: {
                        status: 'SUCCEEDED'
                    }
                },
                interactionModel: {
                    'en-US': {
                        lastUpdateRequest: {
                            status: 'SUCCEEDED'
                        }
                    },
                },
                hostedSkillProvisioning: {
                    lastUpdateRequest: {
                        status: 'SUCCEEDED'
                    }
                }
            }
        };
        beforeEach(() => {
            sandbox = sinon.createSandbox();
            sandbox.stub(apiWrapper, 'callGetSkillStatus');
        });

        it('| Valid status response', (done) => {
            // set up
            apiWrapper.callGetSkillStatus.callsArgOnWith(4, {}, data);

            // call
            hostedSkillController.pollingSkillStatus(TEST_SKILL_ID, false, (error, response) => {
                // verify
                expect(error).equal(null);
                expect(response.provisioning.status).equal('SUCCEEDED');
                expect(response.manifest.status).equal('SUCCEEDED');
                expect(response.interactionModel.status).equal('SUCCEEDED');
                done();
            });
        });

        it('| Invalid status response with 404 status code, expect console error', (done) => {
            // set up
            apiWrapper.callGetSkillStatus.callsArgOnWith(4, {}, { statusCode: 404, body: {} });
            // call
            hostedSkillController.pollingSkillStatus(TEST_SKILL_ID, false, (error, response) => {
                // verify
                expect(INVALID_RESPONSE_ERROR === error);
                expect(response == null);
                done();
            });
        });

        it('| Invalid status response, expect console error', (done) => {
            // set up
            apiWrapper.callGetSkillStatus.callsArgOnWith(4, {}, { statusCode: 400, body: {} });
            // call
            hostedSkillController.pollingSkillStatus(TEST_SKILL_ID, false, (error, response) => {
                // verify
                expect(INVALID_RESPONSE_ERROR === error);
                expect(response == null);
                done();
            });
        });

        it('| Retry reaches maximum', (done) => {
            // set up
            const cacheValue = CONSTANTS.CONFIGURATION.RETRY.GET_PROVISIONING_STATUS.MAX_RETRY;
            CONSTANTS.CONFIGURATION.RETRY.GET_PROVISIONING_STATUS.MAX_RETRY = 1;
            const inProgressData = {
                statusCode: 200,
                body: {
                    manifest: {
                        lastUpdateRequest: {
                            status: 'IN_PROGRESS'
                        }
                    }
                }
            };
            apiWrapper.callGetSkillStatus.callsArgOnWith(4, {}, inProgressData);
            // call
            hostedSkillController.pollingSkillStatus(TEST_SKILL_ID, false, (error, response) => {
                // verify
                expect(response).deep.equal({
                    interactionModel: {},
                    manifest: {
                        errors: undefined,
                        status: 'IN_PROGRESS'
                    },
                    provisioning: {
                        status: undefined
                    }
                });
                expect(error).equal(null);
                CONSTANTS.CONFIGURATION.RETRY.GET_PROVISIONING_STATUS.MAX_RETRY = cacheValue;
                done();
            });
        });

        afterEach(() => {
            sandbox.restore();
        });
    });

    describe('# getHostedSkillPermission method', () => {
        let sandbox;
        const data = {
            statusCode: 200,
            body: {
                status: PERMISSION_RESULT,
                permission: PERMISSION_TYPE,
                actionUrl: '',
            },
        };
        beforeEach(() => {
            sandbox = sinon.createSandbox();
            sandbox.stub(profileHelper, 'resolveVendorId');
            sandbox.stub(apiWrapper, 'getHostedSkillPermission');
        });

        it('| No vendorId found in profile, expect console error', (done) => {
            // set up
            profileHelper.resolveVendorId.withArgs(TEST_PROFILE).returns(null);
            const NO_VENDOR_ID_ERROR = '[Error]: cannot find vendor id associated with the current profile.';
            // call
            hostedSkillController.getHostedSkillPermission(PERMISSION_TYPE, false, (error, response) => {
                // verify
                expect(NO_VENDOR_ID_ERROR === error);
                expect(response == null);
                done();
            });
        });

        it('| getHostedSkillPermission returns invalid data', (done) => {
            // set up
            profileHelper.resolveVendorId.withArgs(TEST_PROFILE).returns(TEST_VENDOR_ID);
            apiWrapper.getHostedSkillPermission.callsArgOnWith(4, {}, {});
            const GET_PERMISSION_ERROR = '[Error]: Cannot parse the response from getHostedSkillPermission request. '
                + 'Please try again with --debug option to see more details.';
            // call
            hostedSkillController.getHostedSkillPermission(PERMISSION_TYPE, false, (error, response) => {
                // verify
                expect(GET_PERMISSION_ERROR === error);
                expect(response == null);
                done();
            });
        });

        it('| getHostedSkillPermission returns valid data', (done) => {
            // set up
            profileHelper.resolveVendorId.withArgs(TEST_PROFILE).returns(TEST_VENDOR_ID);
            apiWrapper.getHostedSkillPermission.callsArgOnWith(4, {}, data);

            // call
            hostedSkillController.getHostedSkillPermission(PERMISSION_TYPE, false, (error, response) => {
                // verify
                expect(response.status).equal(PERMISSION_RESULT);
                expect(response.permission).equal(PERMISSION_TYPE);
                expect(response.actionUrl).equal('');
                done();
            });
        });

        afterEach(() => {
            apiWrapper.getHostedSkillPermission.restore();
            sandbox.restore();
        });
    });
});
